@@ -103,14 +103,13 @@ gdk_arg_context_parse (GdkArgContext *context, gint *argc, gchar ***argv)
 {
   int i, j, k;
 
-  /* Save a copy of the original argc and argv */
   if (argc && argv)
     {
       for (i = 1; i < *argc; i++)
 	{
 	  char *arg;
 	  
-	  if (!(*argv)[i][0] == '-' && (*argv)[i][1] == '-')
+	  if (!((*argv)[i][0] == '-' && (*argv)[i][1] == '-'))
 	    continue;
 	  
 	  arg = (*argv)[i] + 2;
@@ -239,6 +238,7 @@ static GdkArgDesc gdk_args[] = {
   { "class" ,       GDK_ARG_CALLBACK, NULL, gdk_arg_class_cb                   },
   { "name",         GDK_ARG_CALLBACK, NULL, gdk_arg_name_cb                    },
   { "display",      GDK_ARG_STRING,   &_gdk_display_name,     (GdkArgFunc)NULL },
+  { "screen",       GDK_ARG_INT,      &_gdk_screen_number,    (GdkArgFunc)NULL },
 
 #ifdef G_ENABLE_DEBUG
   { "gdk-debug",    GDK_ARG_CALLBACK, NULL, gdk_arg_debug_cb    },
@@ -278,6 +278,8 @@ _gdk_get_command_line_args (int    *argc,
  *
  * You shouldn't call this function explicitely if you are using
  * gtk_init(), gtk_init_check(), gdk_init(), or gdk_init_check().
+ *
+ * Since: 2.2
  **/
 void
 gdk_parse_args (int    *argc,
@@ -291,6 +293,7 @@ gdk_parse_args (int    *argc,
 
   gdk_initialized = TRUE;
 
+  /* Save a copy of the original argc and argv */
   if (argc && argv)
     {
       gdk_argc = *argc;
@@ -299,17 +302,17 @@ gdk_parse_args (int    *argc,
       for (i = 0; i < gdk_argc; i++)
 	gdk_argv[i] = g_strdup ((*argv)[i]);
       gdk_argv[gdk_argc] = NULL;
+    }
 
-      if (*argc > 0)
-	{
-	  gchar *d;
-	  
-	  d = strrchr((*argv)[0], G_DIR_SEPARATOR);
-	  if (d != NULL)
-	    g_set_prgname (d + 1);
-	  else
-	    g_set_prgname ((*argv)[0]);
-	}
+  if (argc && argv && *argc > 0)
+    {
+      gchar *d;
+      
+      d = strrchr((*argv)[0], G_DIR_SEPARATOR);
+      if (d != NULL)
+	g_set_prgname (d + 1);
+      else
+	g_set_prgname ((*argv)[0]);
     }
   else
     {
@@ -355,11 +358,62 @@ gdk_parse_args (int    *argc,
  * to gdk_init() or gdk_parse_args(), if any.
  *
  * Returns: the display name, if specified explicitely, otherwise %NULL
+ *   this string is owned by GTK+ and must not be modified or freed.
+ *
+ * Since: 2.2
  */
-gchar *
+G_CONST_RETURN gchar *
 gdk_get_display_arg_name (void)
 {
-  return _gdk_display_name;
+  if (!_gdk_display_arg_name)
+    {
+      if (_gdk_screen_number >= 0)
+	_gdk_display_arg_name = _gdk_windowing_substitute_screen_number (_gdk_display_name, _gdk_screen_number);
+      else
+	_gdk_display_arg_name = g_strdup (_gdk_display_name);
+   }
+
+   return _gdk_display_arg_name;
+}
+
+/**
+ * gdk_display_open_default_libgtk_only:
+ * 
+ * Opens the default display specified by command line arguments or
+ * environment variables, sets it as the default display, and returns
+ * it.  gdk_parse_args must have been called first. If the default
+ * display has previously been set, simply returns that. An internal
+ * function that should not be used by applications.
+ * 
+ * Return value: the default display, if it could be opened,
+ *   otherwise %NULL.
+ **/
+GdkDisplay *
+gdk_display_open_default_libgtk_only (void)
+{
+  GdkDisplay *display;
+
+  g_return_val_if_fail (gdk_initialized, NULL);
+  
+  display = gdk_display_get_default ();
+  if (display)
+    return display;
+
+  display = gdk_display_open (gdk_get_display_arg_name ());
+
+  if (!display && _gdk_screen_number >= 0)
+    {
+      g_free (_gdk_display_arg_name);
+      _gdk_display_arg_name = g_strdup (_gdk_display_name);
+      
+      display = gdk_display_open (_gdk_display_name);
+    }
+  
+  if (display)
+    gdk_display_manager_set_default_display (gdk_display_manager_get (),
+					     display);
+  
+  return display;
 }
 
 /*
@@ -388,23 +442,9 @@ gboolean
 gdk_init_check (int    *argc,
 		char ***argv)
 {
-  GdkDisplay *display;
-  
   gdk_parse_args (argc, argv);
 
-  if (gdk_display_get_default ())
-    return TRUE;
-    
-  display = gdk_display_open (_gdk_display_name);
-
-  if (display)
-    {
-      gdk_display_manager_set_default_display (gdk_display_manager_get (),
-					       display);
-      return TRUE;
-    }
-  else
-    return FALSE;
+  return gdk_display_open_default_libgtk_only () != NULL;
 }
 
 void
@@ -439,8 +479,6 @@ gdk_init (int *argc, char ***argv)
 void
 gdk_exit (gint errorcode)
 {
-  /* de-initialisation is done by the gdk_exit_funct(),
-     no need to do this here (Alex J.) */
   exit (errorcode);
 }
 
