@@ -28,7 +28,6 @@
 #include "gtklabel.h"
 #include "gtkmain.h"
 #include "gtkmarshalers.h"
-#include "gtksignal.h"
 #include "gtkwindow.h"
 #include "gdk/gdkkeysyms.h"
 #include "gtkclipboard.h"
@@ -131,6 +130,8 @@ static void set_markup                           (GtkLabel      *label,
 static void gtk_label_recalculate                (GtkLabel      *label);
 static void gtk_label_hierarchy_changed          (GtkWidget     *widget,
 						  GtkWidget     *old_toplevel);
+static void gtk_label_screen_changed             (GtkWidget     *widget,
+						  GdkScreen     *old_screen);
 
 static void gtk_label_create_window       (GtkLabel *label);
 static void gtk_label_destroy_window      (GtkLabel *label);
@@ -165,10 +166,10 @@ static gint gtk_label_move_backward_word (GtkLabel        *label,
 static GtkMiscClass *parent_class = NULL;
 
 
-GtkType
+GType
 gtk_label_get_type (void)
 {
-  static GtkType label_type = 0;
+  static GType label_type = 0;
   
   if (!label_type)
     {
@@ -185,7 +186,8 @@ gtk_label_get_type (void)
 	(GInstanceInitFunc) gtk_label_init,
       };
 
-      label_type = g_type_register_static (GTK_TYPE_MISC, "GtkLabel", &label_info, 0);
+      label_type = g_type_register_static (GTK_TYPE_MISC, "GtkLabel",
+					   &label_info, 0);
     }
   
   return label_type;
@@ -202,16 +204,16 @@ add_move_binding (GtkBindingSet  *binding_set,
   
   gtk_binding_entry_add_signal (binding_set, keyval, modmask,
 				"move_cursor", 3,
-				GTK_TYPE_ENUM, step,
+				G_TYPE_ENUM, step,
 				G_TYPE_INT, count,
-                                G_TYPE_BOOLEAN, FALSE);
+				G_TYPE_BOOLEAN, FALSE);
 
   /* Selection-extending version */
   gtk_binding_entry_add_signal (binding_set, keyval, modmask | GDK_SHIFT_MASK,
 				"move_cursor", 3,
-				GTK_TYPE_ENUM, step,
+				G_TYPE_ENUM, step,
 				G_TYPE_INT, count,
-                                G_TYPE_BOOLEAN, TRUE);
+				G_TYPE_BOOLEAN, TRUE);
 }
 
 static void
@@ -222,7 +224,7 @@ gtk_label_class_init (GtkLabelClass *class)
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);
   GtkBindingSet *binding_set;
 
-  parent_class = gtk_type_class (GTK_TYPE_MISC);
+  parent_class = g_type_class_peek_parent (class);
   
   gobject_class->set_property = gtk_label_set_property;
   gobject_class->get_property = gtk_label_get_property;
@@ -244,6 +246,7 @@ gtk_label_class_init (GtkLabelClass *class)
   widget_class->button_release_event = gtk_label_button_release;
   widget_class->motion_notify_event = gtk_label_motion;
   widget_class->hierarchy_changed = gtk_label_hierarchy_changed;
+  widget_class->screen_changed = gtk_label_screen_changed;
   widget_class->mnemonic_activate = gtk_label_mnemonic_activate;
   widget_class->focus = gtk_label_focus;
 
@@ -251,48 +254,55 @@ gtk_label_class_init (GtkLabelClass *class)
   class->copy_clipboard = gtk_label_copy_clipboard;
   
   signals[MOVE_CURSOR] = 
-      gtk_signal_new ("move_cursor",
-                      GTK_RUN_LAST | GTK_RUN_ACTION,
-                      GTK_CLASS_TYPE (object_class),
-                      GTK_SIGNAL_OFFSET (GtkLabelClass, move_cursor),
-                      _gtk_marshal_VOID__ENUM_INT_BOOLEAN,
-                      GTK_TYPE_NONE, 3, GTK_TYPE_MOVEMENT_STEP, GTK_TYPE_INT, GTK_TYPE_BOOL);
+    g_signal_new ("move_cursor",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+		  G_STRUCT_OFFSET (GtkLabelClass, move_cursor),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__ENUM_INT_BOOLEAN,
+		  G_TYPE_NONE, 3,
+		  GTK_TYPE_MOVEMENT_STEP,
+		  G_TYPE_INT,
+		  G_TYPE_BOOLEAN);
   
   signals[COPY_CLIPBOARD] =
-    gtk_signal_new ("copy_clipboard",
-                    GTK_RUN_LAST | GTK_RUN_ACTION,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkLabelClass, copy_clipboard),
-                    _gtk_marshal_VOID__VOID,
-                    GTK_TYPE_NONE, 0);
+    g_signal_new ("copy_clipboard",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+		  G_STRUCT_OFFSET (GtkLabelClass, copy_clipboard),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
   
   signals[POPULATE_POPUP] =
-    gtk_signal_new ("populate_popup",
-		    GTK_RUN_LAST,
-		    GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (GtkLabelClass, populate_popup),
-		    _gtk_marshal_VOID__OBJECT,
-		    GTK_TYPE_NONE, 1, GTK_TYPE_MENU);
+    g_signal_new ("populate_popup",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_LAST,
+		  G_STRUCT_OFFSET (GtkLabelClass, populate_popup),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__OBJECT,
+		  G_TYPE_NONE, 1,
+		  GTK_TYPE_MENU);
 
-  g_object_class_install_property (G_OBJECT_CLASS(object_class),
+  g_object_class_install_property (gobject_class,
                                    PROP_LABEL,
                                    g_param_spec_string ("label",
                                                         _("Label"),
-                                                        _("The text of the label."),
+                                                        _("The text of the label"),
                                                         NULL,
                                                         G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
 				   PROP_ATTRIBUTES,
 				   g_param_spec_boxed ("attributes",
 						       _("Attributes"),
-						       _("A list of style attributes to apply to the text of the label."),
+						       _("A list of style attributes to apply to the text of the label"),
 						       PANGO_TYPE_ATTR_LIST,
 						       G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
                                    PROP_USE_MARKUP,
                                    g_param_spec_boolean ("use_markup",
 							 _("Use markup"),
-							 _("The text of the label includes XML markup. See pango_parse_markup()."),
+							 _("The text of the label includes XML markup. See pango_parse_markup()"),
                                                         FALSE,
                                                         G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
@@ -307,7 +317,7 @@ gtk_label_class_init (GtkLabelClass *class)
 				   PROP_JUSTIFY,
                                    g_param_spec_enum ("justify",
                                                       _("Justification"),
-                                                      _("The alignment of the lines in the text of the label relative to each other. This does NOT affect the alignment of the label within its allocation. See GtkMisc::xalign for that."),
+                                                      _("The alignment of the lines in the text of the label relative to each other. This does NOT affect the alignment of the label within its allocation. See GtkMisc::xalign for that"),
 						      GTK_TYPE_JUSTIFICATION,
 						      GTK_JUSTIFY_LEFT,
                                                       G_PARAM_READWRITE));
@@ -316,7 +326,7 @@ gtk_label_class_init (GtkLabelClass *class)
                                    PROP_PATTERN,
                                    g_param_spec_string ("pattern",
                                                         _("Pattern"),
-                                                        _("A string with _ characters in positions correspond to characters in the text to underline."),
+                                                        _("A string with _ characters in positions correspond to characters in the text to underline"),
                                                         NULL,
                                                         G_PARAM_WRITABLE));
 
@@ -324,21 +334,21 @@ gtk_label_class_init (GtkLabelClass *class)
                                    PROP_WRAP,
                                    g_param_spec_boolean ("wrap",
                                                         _("Line wrap"),
-                                                        _("If set, wrap lines if the text becomes too wide."),
+                                                        _("If set, wrap lines if the text becomes too wide"),
                                                         FALSE,
                                                         G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
                                    PROP_SELECTABLE,
                                    g_param_spec_boolean ("selectable",
                                                         _("Selectable"),
-                                                        _("Whether the label text can be selected with the mouse."),
+                                                        _("Whether the label text can be selected with the mouse"),
                                                         FALSE,
                                                         G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
                                    PROP_MNEMONIC_KEYVAL,
                                    g_param_spec_uint ("mnemonic_keyval",
 						      _("Mnemonic key"),
-						      _("The mnemonic accelerator key for this label."),
+						      _("The mnemonic accelerator key for this label"),
 						      0,
 						      G_MAXUINT,
 						      GDK_VoidSymbol,
@@ -348,7 +358,7 @@ gtk_label_class_init (GtkLabelClass *class)
                                    g_param_spec_object ("mnemonic_widget",
 							_("Mnemonic widget"),
 							_("The widget to be activated when the label's mnemonic "
-							  "key is pressed."),
+							  "key is pressed"),
 							GTK_TYPE_WIDGET,
 							G_PARAM_READWRITE));
 
@@ -356,7 +366,7 @@ gtk_label_class_init (GtkLabelClass *class)
                                    PROP_CURSOR_POSITION,
                                    g_param_spec_int ("cursor_position",
                                                      _("Cursor Position"),
-                                                     _("The current position of the insertion cursor in chars."),
+                                                     _("The current position of the insertion cursor in chars"),
                                                      0,
                                                      G_MAXINT,
                                                      0,
@@ -366,7 +376,7 @@ gtk_label_class_init (GtkLabelClass *class)
                                    PROP_SELECTION_BOUND,
                                    g_param_spec_int ("selection_bound",
                                                      _("Selection Bound"),
-                                                     _("The position of the opposite end of the selection from the cursor in chars."),
+                                                     _("The position of the opposite end of the selection from the cursor in chars"),
                                                      0,
                                                      G_MAXINT,
                                                      0,
@@ -457,10 +467,8 @@ gtk_label_set_property (GObject      *object,
 			GParamSpec   *pspec)
 {
   GtkLabel *label;
-  guint last_keyval;
 
   label = GTK_LABEL (object);
-  last_keyval = label->mnemonic_keyval;
   
   switch (prop_id)
     {
@@ -601,7 +609,7 @@ gtk_label_new (const gchar *str)
 {
   GtkLabel *label;
   
-  label = gtk_type_new (GTK_TYPE_LABEL);
+  label = g_object_new (GTK_TYPE_LABEL, NULL);
 
   if (str && *str)
     gtk_label_set_text (label, str);
@@ -637,7 +645,7 @@ gtk_label_new_with_mnemonic (const gchar *str)
 {
   GtkLabel *label;
   
-  label = gtk_type_new (GTK_TYPE_LABEL);
+  label = g_object_new (GTK_TYPE_LABEL, NULL);
 
   if (str && *str)
     gtk_label_set_text_with_mnemonic (label, str);
@@ -674,7 +682,7 @@ gtk_label_mnemonic_activate (GtkWidget *widget,
 
   /* barf if there was nothing to activate */
   g_warning ("Couldn't find a target for a mnemonic activation.");
-  gdk_beep ();
+  gdk_display_beep (gtk_widget_get_display (widget));
   
   return FALSE;
 }
@@ -711,10 +719,26 @@ gtk_label_hierarchy_changed (GtkWidget *widget,
 			     GtkWidget *old_toplevel)
 {
   GtkLabel *label = GTK_LABEL (widget);
-  
+
   gtk_label_setup_mnemonic (label, label->mnemonic_keyval);
 }
 
+static void
+gtk_label_screen_changed (GtkWidget *widget,
+			  GdkScreen *old_screen)
+{
+  gtk_label_clear_layout (GTK_LABEL (widget));
+}
+
+static void
+label_mnemonic_widget_weak_notify (gpointer      data,
+				   GObject      *where_the_object_was)
+{
+  GtkLabel *label = data;
+
+  label->mnemonic_widget = NULL;
+  g_object_notify (G_OBJECT (label), "mnemonic_widget");
+}
 
 /**
  * gtk_label_set_mnemonic_widget:
@@ -744,10 +768,14 @@ gtk_label_set_mnemonic_widget (GtkLabel  *label,
     g_return_if_fail (GTK_IS_WIDGET (widget));
 
   if (label->mnemonic_widget)
-    gtk_widget_unref (label->mnemonic_widget);
+    g_object_weak_unref (G_OBJECT (label->mnemonic_widget),
+			 label_mnemonic_widget_weak_notify,
+			 label);
   label->mnemonic_widget = widget;
   if (label->mnemonic_widget)
-    gtk_widget_ref (label->mnemonic_widget);
+    g_object_weak_ref (G_OBJECT (label->mnemonic_widget),
+		       label_mnemonic_widget_weak_notify,
+		       label);
   
   g_object_notify (G_OBJECT (label), "mnemonic_widget");
 }
@@ -926,7 +954,8 @@ gtk_label_set_text (GtkLabel    *label,
  * 
  * Sets a #PangoAttrList; the attributes in the list are applied to the
  * label text. The attributes set with this function will be ignored
- * if label->use_underline or label->use_markup is %TRUE.
+ * if the "use_underline" property or the "use_markup" property
+ * is %TRUE.
  **/
 void
 gtk_label_set_attributes (GtkLabel         *label,
@@ -1304,7 +1333,7 @@ gtk_label_finalize (GObject *object)
   g_free (label->text);
 
   if (label->layout)
-    g_object_unref (G_OBJECT (label->layout));
+    g_object_unref (label->layout);
 
   if (label->attrs)
     pango_attr_list_unref (label->attrs);
@@ -1322,7 +1351,7 @@ gtk_label_clear_layout (GtkLabel *label)
 {
   if (label->layout)
     {
-      g_object_unref (G_OBJECT (label->layout));
+      g_object_unref (label->layout);
       label->layout = NULL;
     }
 }
@@ -1426,6 +1455,7 @@ gtk_label_ensure_layout (GtkLabel *label)
 	    pango_layout_set_width (label->layout, aux_info->width * PANGO_SCALE);
 	  else
 	    {
+	      GdkScreen *screen = gtk_widget_get_screen (GTK_WIDGET (label));
 	      gint wrap_width;
 	      
 	      pango_layout_set_width (label->layout, -1);
@@ -1439,7 +1469,7 @@ gtk_label_ensure_layout (GtkLabel *label)
 	      wrap_width = get_label_wrap_width (label);
 	      width = MIN (width, wrap_width);
 	      width = MIN (width,
-			   PANGO_SCALE * (gdk_screen_width () + 1) / 2);
+			   PANGO_SCALE * (gdk_screen_get_width (screen) + 1) / 2);
 	      
 	      pango_layout_set_width (label->layout, width);
 	      pango_layout_get_extents (label->layout, NULL, &logical_rect);
@@ -1644,15 +1674,21 @@ get_layout_location (GtkLabel  *label,
     xalign = misc->xalign;
   else
     xalign = 1.0 - misc->xalign;
-  
-  x = floor (widget->allocation.x + (gint)misc->xpad
-             + ((widget->allocation.width - widget->requisition.width) * xalign)
-             + 0.5);
-  
+
+  x = floor (widget->allocation.x + (gint)misc->xpad +
+	     xalign * (widget->allocation.width - widget->requisition.width)
+	     + 0.5);
+
+  if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR)
+    x = MAX (x, widget->allocation.x + misc->xpad);
+  else
+    x = MIN (x,
+	     widget->allocation.x + widget->allocation.width -
+	     widget->requisition.width - misc->xpad);
+
   y = floor (widget->allocation.y + (gint)misc->ypad 
-             + ((widget->allocation.height - widget->requisition.height) * misc->yalign)
-             + 0.5);
-  
+             + MAX (((widget->allocation.height - widget->requisition.height) * misc->yalign)
+		    + 0.5, 0));
 
   if (xp)
     *xp = x;
@@ -1683,7 +1719,7 @@ gtk_label_draw_cursor (GtkLabel  *label, gint xoffset, gint yoffset)
       GdkGC *gc;
 
       keymap_direction =
-	(gdk_keymap_get_direction (gdk_keymap_get_default ()) == PANGO_DIRECTION_LTR) ?
+	(gdk_keymap_get_direction (gdk_keymap_get_for_display (gtk_widget_get_display (widget))) == PANGO_DIRECTION_LTR) ?
 	GTK_TEXT_DIR_LTR : GTK_TEXT_DIR_RTL;
 
       widget_direction = gtk_widget_get_direction (widget);
@@ -2290,7 +2326,8 @@ gtk_label_create_window (GtkLabel *label)
   attributes.window_type = GDK_WINDOW_TEMP;
   attributes.wclass = GDK_INPUT_ONLY;
   attributes.override_redirect = TRUE;
-  attributes.cursor = gdk_cursor_new (GDK_XTERM);
+  attributes.cursor = gdk_cursor_new_for_display (gtk_widget_get_display (widget),
+						  GDK_XTERM);
   attributes.event_mask = gtk_widget_get_events (widget) |
     GDK_BUTTON_PRESS_MASK        |
     GDK_BUTTON_RELEASE_MASK      |
@@ -2476,7 +2513,8 @@ gtk_label_select_region_index (GtkLabel *label,
       label->select_info->selection_anchor = anchor_index;
       label->select_info->selection_end = end_index;
 
-      clipboard = gtk_clipboard_get (GDK_SELECTION_PRIMARY);      
+      clipboard = gtk_widget_get_clipboard (GTK_WIDGET (label),
+					    GDK_SELECTION_PRIMARY);      
       
       if (anchor_index != end_index)
         {
@@ -2745,8 +2783,9 @@ get_better_cursor (GtkLabel *label,
 		   gint      *x,
 		   gint      *y)
 {
+  GdkKeymap *keymap = gdk_keymap_get_for_display (gtk_widget_get_display (GTK_WIDGET (label)));
   GtkTextDirection keymap_direction =
-    (gdk_keymap_get_direction (gdk_keymap_get_default ()) == PANGO_DIRECTION_LTR) ?
+    (gdk_keymap_get_direction (keymap) == PANGO_DIRECTION_LTR) ?
     GTK_TEXT_DIR_LTR : GTK_TEXT_DIR_RTL;
   GtkTextDirection widget_direction = gtk_widget_get_direction (GTK_WIDGET (label));
   gboolean split_cursor;
@@ -2850,8 +2889,9 @@ gtk_label_move_visually (GtkLabel *label,
 	strong = TRUE;
       else
 	{
+	  GdkKeymap *keymap = gdk_keymap_get_for_display (gtk_widget_get_display (GTK_WIDGET (label)));
 	  GtkTextDirection keymap_direction =
-	    (gdk_keymap_get_direction (gdk_keymap_get_default ()) == PANGO_DIRECTION_LTR) ?
+	    (gdk_keymap_get_direction (keymap) == PANGO_DIRECTION_LTR) ?
 	    GTK_TEXT_DIR_LTR : GTK_TEXT_DIR_RTL;
 
 	  strong = keymap_direction == gtk_widget_get_direction (GTK_WIDGET (label));
@@ -3064,7 +3104,8 @@ gtk_label_copy_clipboard (GtkLabel *label)
         start = len;
 
       if (start != end)
-	gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD),
+	gtk_clipboard_set_text (gtk_widget_get_clipboard (GTK_WIDGET (label),
+							  GDK_SELECTION_CLIPBOARD),
 				label->text + start, end - start);
     }
 }
@@ -3081,8 +3122,8 @@ static void
 activate_cb (GtkWidget *menuitem,
 	     GtkLabel  *label)
 {
-  const gchar *signal = gtk_object_get_data (GTK_OBJECT (menuitem), "gtk-signal");
-  gtk_signal_emit_by_name (GTK_OBJECT (label), signal);
+  const gchar *signal = g_object_get_data (G_OBJECT (menuitem), "gtk-signal");
+  g_signal_emit_by_name (label, signal);
 }
 
 static void
@@ -3094,9 +3135,9 @@ append_action_signal (GtkLabel     *label,
 {
   GtkWidget *menuitem = gtk_image_menu_item_new_from_stock (stock_id, NULL);
 
-  gtk_object_set_data (GTK_OBJECT (menuitem), "gtk-signal", (char *)signal);
-  gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
-		      GTK_SIGNAL_FUNC (activate_cb), label);
+  g_object_set_data (G_OBJECT (menuitem), "gtk-signal", (char *)signal);
+  g_signal_connect (menuitem, "activate",
+		    G_CALLBACK (activate_cb), label);
 
   gtk_widget_set_sensitive (menuitem, sensitive);
   
@@ -3125,6 +3166,7 @@ popup_position_func (GtkMenu   *menu,
   GtkLabel *label;
   GtkWidget *widget;
   GtkRequisition req;
+  GdkScreen *screen;
   
   label = GTK_LABEL (user_data);  
   widget = GTK_WIDGET (label);
@@ -3133,7 +3175,8 @@ popup_position_func (GtkMenu   *menu,
     return;
   
   g_return_if_fail (GTK_WIDGET_REALIZED (label));
-
+  
+  screen = gtk_widget_get_screen (widget);
   gdk_window_get_origin (widget->window, x, y);      
 
   gtk_widget_size_request (label->select_info->popup_menu, &req);
@@ -3141,8 +3184,8 @@ popup_position_func (GtkMenu   *menu,
   *x += widget->allocation.width / 2;
   *y += widget->allocation.height;
 
-  *x = CLAMP (*x, 0, MAX (0, gdk_screen_width () - req.width));
-  *y = CLAMP (*y, 0, MAX (0, gdk_screen_height () - req.height));
+  *x = CLAMP (*x, 0, MAX (0, gdk_screen_get_width (screen) - req.width));
+  *y = CLAMP (*y, 0, MAX (0, gdk_screen_get_height (screen) - req.height));
 }
 
 
@@ -3177,8 +3220,8 @@ gtk_label_do_popup (GtkLabel       *label,
                         FALSE);
   
   menuitem = gtk_menu_item_new_with_label (_("Select All"));
-  gtk_signal_connect_object (GTK_OBJECT (menuitem), "activate",
-			     GTK_SIGNAL_FUNC (gtk_label_select_all), label);
+  g_signal_connect_swapped (menuitem, "activate",
+			    G_CALLBACK (gtk_label_select_all), label);
   gtk_widget_show (menuitem);
   gtk_menu_shell_append (GTK_MENU_SHELL (label->select_info->popup_menu), menuitem);
 
@@ -3192,9 +3235,10 @@ gtk_label_do_popup (GtkLabel       *label,
   gtk_widget_set_sensitive (menuitem, FALSE);
   gtk_menu_shell_append (GTK_MENU_SHELL (label->select_info->popup_menu), menuitem);
 
-  gtk_signal_emit (GTK_OBJECT (label),
-                   signals[POPULATE_POPUP],
-                   label->select_info->popup_menu);
+  g_signal_emit (label,
+		 signals[POPULATE_POPUP],
+		 0,
+		 label->select_info->popup_menu);
   
   if (event)
     gtk_menu_popup (GTK_MENU (label->select_info->popup_menu), NULL, NULL,
