@@ -24,6 +24,8 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#undef GTK_DISABLE_DEPRECATED
+
 #include "gtkbindings.h"
 #include "gtklabel.h"
 #include "gtklistitem.h"
@@ -390,7 +392,8 @@ gtk_list_item_new_with_label (const gchar *label)
   list_item = gtk_list_item_new ();
   label_widget = gtk_label_new (label);
   gtk_misc_set_alignment (GTK_MISC (label_widget), 0.0, 0.5);
-
+  gtk_misc_set_padding (GTK_MISC (label_widget), 0, 1);
+  
   gtk_container_add (GTK_CONTAINER (list_item), label_widget);
   gtk_widget_show (label_widget);
 
@@ -457,15 +460,22 @@ gtk_list_item_size_request (GtkWidget      *widget,
 {
   GtkBin *bin;
   GtkRequisition child_requisition;
+  gint focus_width;
+  gint focus_pad;
 
   g_return_if_fail (GTK_IS_LIST_ITEM (widget));
   g_return_if_fail (requisition != NULL);
 
   bin = GTK_BIN (widget);
+  gtk_widget_style_get (widget,
+			"focus-line-width", &focus_width,
+			"focus-padding", &focus_pad,
+			NULL);
 
-  requisition->width = (GTK_CONTAINER (widget)->border_width +
-			widget->style->xthickness) * 2;
-  requisition->height = GTK_CONTAINER (widget)->border_width * 2;
+  requisition->width = 2 * (GTK_CONTAINER (widget)->border_width +
+			    widget->style->xthickness + focus_width + focus_pad - 1);
+  requisition->height = 2 * (GTK_CONTAINER (widget)->border_width +
+			     focus_width + focus_pad - 1);
 
   if (bin->child && GTK_WIDGET_VISIBLE (bin->child))
     {
@@ -557,17 +567,13 @@ gtk_list_item_expose (GtkWidget      *widget,
       if (GTK_WIDGET_HAS_FOCUS (widget))
         {
           if (GTK_IS_LIST (widget->parent) && GTK_LIST (widget->parent)->add_mode)
-            gtk_paint_focus (widget->style, widget->window,
+            gtk_paint_focus (widget->style, widget->window, GTK_WIDGET_STATE (widget),
                              NULL, widget, "add-mode",
-                             0, 0,
-                             widget->allocation.width - 1,
-                             widget->allocation.height - 1);
+                             0, 0, widget->allocation.width, widget->allocation.height);
           else
-            gtk_paint_focus (widget->style, widget->window,
+            gtk_paint_focus (widget->style, widget->window, GTK_WIDGET_STATE (widget),
                              NULL, widget, NULL,
-                             0, 0,
-                             widget->allocation.width - 1,
-                             widget->allocation.height - 1);
+                             0, 0, widget->allocation.width, widget->allocation.height);
         }
     }
 
