@@ -35,7 +35,6 @@
 #endif
 
 #include "gtkprogressbar.h"
-#include "gtksignal.h"
 #include "gtkintl.h"
 
 
@@ -79,27 +78,39 @@ static void gtk_progress_bar_real_update   (GtkProgress         *progress);
 static void gtk_progress_bar_paint         (GtkProgress         *progress);
 static void gtk_progress_bar_act_mode_enter (GtkProgress        *progress);
 
+static void gtk_progress_bar_set_bar_style_internal       (GtkProgressBar *pbar,
+							   GtkProgressBarStyle style);
+static void gtk_progress_bar_set_discrete_blocks_internal (GtkProgressBar *pbar,
+							   guint           blocks);
+static void gtk_progress_bar_set_activity_step_internal   (GtkProgressBar *pbar,
+							   guint           step);
+static void gtk_progress_bar_set_activity_blocks_internal (GtkProgressBar *pbar,
+							   guint           blocks);
 
-GtkType
+
+GType
 gtk_progress_bar_get_type (void)
 {
-  static GtkType progress_bar_type = 0;
+  static GType progress_bar_type = 0;
 
   if (!progress_bar_type)
     {
-      static const GtkTypeInfo progress_bar_info =
+      static const GTypeInfo progress_bar_info =
       {
-	"GtkProgressBar",
-	sizeof (GtkProgressBar),
 	sizeof (GtkProgressBarClass),
-	(GtkClassInitFunc) gtk_progress_bar_class_init,
-	(GtkObjectInitFunc) gtk_progress_bar_init,
-        /* reserved_1 */ NULL,
-        /* reserved_2 */ NULL,
-        (GtkClassInitFunc) NULL
+	NULL,		/* base_init */
+	NULL,		/* base_finalize */
+	(GClassInitFunc) gtk_progress_bar_class_init,
+	NULL,		/* class_finalize */
+	NULL,		/* class_data */
+	sizeof (GtkProgressBar),
+	0,		/* n_preallocs */
+	(GInstanceInitFunc) gtk_progress_bar_init,
       };
 
-      progress_bar_type = gtk_type_unique (GTK_TYPE_PROGRESS, &progress_bar_info);
+      progress_bar_type =
+	g_type_register_static (GTK_TYPE_PROGRESS, "GtkProgressBar",
+				&progress_bar_info, 0);
     }
 
   return progress_bar_type;
@@ -245,16 +256,16 @@ gtk_progress_bar_set_property (GObject      *object,
       gtk_progress_bar_set_orientation (pbar, g_value_get_enum (value));
       break;
     case PROP_BAR_STYLE:
-      gtk_progress_bar_set_bar_style (pbar, g_value_get_enum (value));
+      gtk_progress_bar_set_bar_style_internal (pbar, g_value_get_enum (value));
       break;
     case PROP_ACTIVITY_STEP:
-      gtk_progress_bar_set_activity_step (pbar, g_value_get_uint (value));
+      gtk_progress_bar_set_activity_step_internal (pbar, g_value_get_uint (value));
       break;
     case PROP_ACTIVITY_BLOCKS:
-      gtk_progress_bar_set_activity_blocks (pbar, g_value_get_uint (value));
+      gtk_progress_bar_set_activity_blocks_internal (pbar, g_value_get_uint (value));
       break;
     case PROP_DISCRETE_BLOCKS:
-      gtk_progress_bar_set_discrete_blocks (pbar, g_value_get_uint (value));
+      gtk_progress_bar_set_discrete_blocks_internal (pbar, g_value_get_uint (value));
       break;
     case PROP_FRACTION:
       gtk_progress_bar_set_fraction (pbar, g_value_get_double (value));
@@ -467,7 +478,7 @@ gtk_progress_bar_size_request (GtkWidget      *widget,
       layout = gtk_widget_create_pango_layout (widget, buf);
       pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
 	  
-      g_object_unref (G_OBJECT (layout));
+      g_object_unref (layout);
       g_free (buf);
     }
   
@@ -516,18 +527,28 @@ gtk_progress_bar_act_mode_enter (GtkProgress *progress)
   GtkProgressBar *pbar;
   GtkWidget *widget;
   gint size;
+  GtkProgressBarOrientation orientation;
 
   pbar = GTK_PROGRESS_BAR (progress);
   widget = GTK_WIDGET (progress);
 
+  orientation = pbar->orientation;
+  if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL) 
+    {
+      if (pbar->orientation == GTK_PROGRESS_LEFT_TO_RIGHT)
+	orientation = GTK_PROGRESS_RIGHT_TO_LEFT;
+      else if (pbar->orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
+	orientation = GTK_PROGRESS_LEFT_TO_RIGHT;
+    }
+  
   /* calculate start pos */
 
-  if (pbar->orientation == GTK_PROGRESS_LEFT_TO_RIGHT ||
-      pbar->orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
+  if (orientation == GTK_PROGRESS_LEFT_TO_RIGHT ||
+      orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
     {
       size = MAX (2, widget->allocation.width / pbar->activity_blocks);
 
-      if (pbar->orientation == GTK_PROGRESS_LEFT_TO_RIGHT)
+      if (orientation == GTK_PROGRESS_LEFT_TO_RIGHT)
 	{
 	  pbar->activity_pos = widget->style->xthickness;
 	  pbar->activity_dir = 0;
@@ -544,7 +565,7 @@ gtk_progress_bar_act_mode_enter (GtkProgress *progress)
     {
       size = MAX (2, widget->allocation.height / pbar->activity_blocks);
 
-      if (pbar->orientation == GTK_PROGRESS_TOP_TO_BOTTOM)
+      if (orientation == GTK_PROGRESS_TOP_TO_BOTTOM)
 	{
 	  pbar->activity_pos = widget->style->ythickness;
 	  pbar->activity_dir = 0;
@@ -559,35 +580,254 @@ gtk_progress_bar_act_mode_enter (GtkProgress *progress)
     }
 }
 
+static void
+gtk_progress_bar_paint_activity (GtkProgressBar            *pbar,
+				 GtkProgressBarOrientation  orientation)
+{
+  GtkWidget *widget = GTK_WIDGET (pbar);
+  GtkProgress *progress = GTK_PROGRESS (pbar);
+  gint x, y, w, h;
+
+  switch (orientation)
+    {
+    case GTK_PROGRESS_LEFT_TO_RIGHT:
+    case GTK_PROGRESS_RIGHT_TO_LEFT:
+      x = pbar->activity_pos;
+      y = widget->style->ythickness;
+      w = MAX (2, widget->allocation.width / pbar->activity_blocks);
+      h = widget->allocation.height - 2 * widget->style->ythickness;
+      break;
+
+    case GTK_PROGRESS_TOP_TO_BOTTOM:
+    case GTK_PROGRESS_BOTTOM_TO_TOP:
+      x = widget->style->xthickness;
+      y = pbar->activity_pos;
+      w = widget->allocation.width - 2 * widget->style->xthickness;
+      h = MAX (2, widget->allocation.height / pbar->activity_blocks);
+      break;
+
+    default:
+      return;
+      break;
+    }
+
+  gtk_paint_box (widget->style,
+		 progress->offscreen_pixmap,
+		 GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
+		 NULL, widget, "bar",
+		 x, y, w, h);
+}
+
+static void
+gtk_progress_bar_paint_continuous (GtkProgressBar            *pbar,
+				   gint                       amount,
+				   GtkProgressBarOrientation  orientation)
+{
+  GtkWidget *widget = GTK_WIDGET (pbar);
+  gint x, y, w, h;
+
+  if (amount <= 0)
+    return;
+
+  switch (orientation)
+    {
+    case GTK_PROGRESS_LEFT_TO_RIGHT:
+    case GTK_PROGRESS_RIGHT_TO_LEFT:
+      w = amount;
+      h = widget->allocation.height - widget->style->ythickness * 2;
+      y = widget->style->ythickness;
+      
+      x = widget->style->xthickness;
+      if (orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
+	x = widget->allocation.width - amount - x;
+      break;
+      
+    case GTK_PROGRESS_TOP_TO_BOTTOM:
+    case GTK_PROGRESS_BOTTOM_TO_TOP:
+      w = widget->allocation.width - widget->style->xthickness * 2;
+      h = amount;
+      x = widget->style->xthickness;
+      
+      y = widget->style->ythickness;
+      if (orientation == GTK_PROGRESS_BOTTOM_TO_TOP)
+	y = widget->allocation.height - amount - y;
+      break;
+      
+    default:
+      return;
+      break;
+    }
+  
+  gtk_paint_box (widget->style,
+		 GTK_PROGRESS (pbar)->offscreen_pixmap,
+		 GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
+		 NULL, widget, "bar",
+		 x, y, w, h);
+}
+
+static void
+gtk_progress_bar_paint_discrete (GtkProgressBar            *pbar,
+				 GtkProgressBarOrientation  orientation)
+{
+  GtkWidget *widget = GTK_WIDGET (pbar);
+  gint i;
+
+  for (i = 0; i <= pbar->in_block; i++)
+    {
+      gint x, y, w, h, space;
+
+      switch (orientation)
+	{
+	case GTK_PROGRESS_LEFT_TO_RIGHT:
+	case GTK_PROGRESS_RIGHT_TO_LEFT:
+	  space = widget->allocation.width - 2 * widget->style->xthickness;
+	  
+	  x = widget->style->xthickness + (i * space) / pbar->blocks;
+	  y = widget->style->ythickness;
+	  w = widget->style->xthickness + ((i + 1) * space) / pbar->blocks - x;
+	  h = widget->allocation.height - 2 * widget->style->ythickness;
+
+	  if (orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
+	    x = widget->allocation.width - w - x;
+	  break;
+	  
+	case GTK_PROGRESS_TOP_TO_BOTTOM:
+	case GTK_PROGRESS_BOTTOM_TO_TOP:
+	  space = widget->allocation.height - 2 * widget->style->ythickness;
+	  
+	  x = widget->style->xthickness;
+	  y = widget->style->ythickness + (i * space) / pbar->blocks;
+	  w = widget->allocation.width - 2 * widget->style->xthickness;
+	  h = widget->style->ythickness + ((i + 1) * space) / pbar->blocks - y;
+	  
+	  if (orientation == GTK_PROGRESS_BOTTOM_TO_TOP)
+	    y = widget->allocation.height - h - y;
+	  break;
+
+	default:
+	  return;
+	  break;
+	}
+      
+      gtk_paint_box (widget->style,
+		     GTK_PROGRESS (pbar)->offscreen_pixmap,
+		     GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
+		     NULL, widget, "bar",
+		     x, y, w, h);
+    }
+}
+
+static void
+gtk_progress_bar_paint_text (GtkProgressBar            *pbar,
+			     gint			amount,
+			     GtkProgressBarOrientation  orientation)
+{
+  GtkProgress *progress = GTK_PROGRESS (pbar);
+  GtkWidget *widget = GTK_WIDGET (pbar);
+  
+  gint x;
+  gint y;
+  gchar *buf;
+  GdkRectangle rect;
+  PangoLayout *layout;
+  PangoRectangle logical_rect;
+  GdkRectangle prelight_clip, normal_clip;
+  
+  buf = gtk_progress_get_current_text (progress);
+  
+  layout = gtk_widget_create_pango_layout (widget, buf);
+  pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
+  
+  x = widget->style->xthickness + 1 +
+    (widget->allocation.width - 2 * widget->style->xthickness -
+     2 - logical_rect.width)
+    * progress->x_align; 
+
+  y = widget->style->ythickness + 1 +
+    (widget->allocation.height - 2 * widget->style->ythickness -
+     2 - logical_rect.height)
+    * progress->y_align;
+
+  rect.x = widget->style->xthickness;
+  rect.y = widget->style->ythickness;
+  rect.width = widget->allocation.width - 2 * widget->style->xthickness;
+  rect.height = widget->allocation.height - 2 * widget->style->ythickness;
+
+  prelight_clip = normal_clip = rect;
+
+  switch (orientation)
+    {
+    case GTK_PROGRESS_LEFT_TO_RIGHT:
+      prelight_clip.width = amount;
+      normal_clip.x += amount;
+      normal_clip.width -= amount;
+      break;
+      
+    case GTK_PROGRESS_RIGHT_TO_LEFT:
+      normal_clip.width -= amount;
+      prelight_clip.x += normal_clip.width;
+      prelight_clip.width -= normal_clip.width;
+      break;
+       
+    case GTK_PROGRESS_TOP_TO_BOTTOM:
+      prelight_clip.height = amount;
+      normal_clip.y += amount;
+      normal_clip.height -= amount;
+      break;
+      
+    case GTK_PROGRESS_BOTTOM_TO_TOP:
+      normal_clip.height -= amount;
+      prelight_clip.y += normal_clip.height;
+      prelight_clip.height -= normal_clip.height;
+      break;
+    }
+  
+  gtk_paint_layout (widget->style,
+		    progress->offscreen_pixmap,
+		    GTK_STATE_PRELIGHT,
+		    FALSE,
+		    &prelight_clip,
+		    widget,
+		    "progressbar",
+		    x, y,
+		    layout);
+  
+  gtk_paint_layout (widget->style,
+		    progress->offscreen_pixmap,
+		    GTK_STATE_NORMAL,
+		    FALSE,
+		    &normal_clip,
+		    widget,
+		    "progressbar",
+		    x, y,
+		    layout);
+
+  g_object_unref (layout);
+  g_free (buf);
+}
+
 static void
 gtk_progress_bar_paint (GtkProgress *progress)
 {
   GtkProgressBar *pbar;
   GtkWidget *widget;
-  gint amount;
-  gint block_delta = 0;
-  gint space = 0;
-  gint i;
-  gint x;
-  gint y;
-  gdouble percentage;
-  gint size;
+
+  GtkProgressBarOrientation orientation;
 
   g_return_if_fail (GTK_IS_PROGRESS_BAR (progress));
 
   pbar = GTK_PROGRESS_BAR (progress);
   widget = GTK_WIDGET (progress);
 
-  if (pbar->orientation == GTK_PROGRESS_LEFT_TO_RIGHT ||
-      pbar->orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
-    space = widget->allocation.width -
-      2 * widget->style->xthickness;
-  else
-    space = widget->allocation.height -
-      2 * widget->style->ythickness;
-
-  percentage = gtk_progress_get_current_percentage (progress);
-
+  orientation = pbar->orientation;
+  if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL) 
+    {
+      if (pbar->orientation == GTK_PROGRESS_LEFT_TO_RIGHT)
+	orientation = GTK_PROGRESS_RIGHT_TO_LEFT;
+      else if (pbar->orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
+	orientation = GTK_PROGRESS_LEFT_TO_RIGHT;
+    }
+ 
   if (progress->offscreen_pixmap)
     {
       gtk_paint_box (widget->style,
@@ -600,248 +840,94 @@ gtk_progress_bar_paint (GtkProgress *progress)
       
       if (progress->activity_mode)
 	{
-	  if (pbar->orientation == GTK_PROGRESS_LEFT_TO_RIGHT ||
-	      pbar->orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
+	  gtk_progress_bar_paint_activity (pbar, orientation);
+	}
+      else
+	{
+	  gint amount;
+	  gint space;
+	  
+	  if (orientation == GTK_PROGRESS_LEFT_TO_RIGHT ||
+	      orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
+	    space = widget->allocation.width - 2 * widget->style->xthickness;
+	  else
+	    space = widget->allocation.height - 2 * widget->style->ythickness;
+	  
+	  amount = space *
+	    gtk_progress_get_current_percentage (GTK_PROGRESS (pbar));
+	  
+	  if (pbar->bar_style == GTK_PROGRESS_CONTINUOUS)
 	    {
-	      size = MAX (2, widget->allocation.width / pbar->activity_blocks);
-	      
-	      gtk_paint_box (widget->style,
-			     progress->offscreen_pixmap,
-			     GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-			     NULL, widget, "bar",
-			     pbar->activity_pos,
-			     widget->style->ythickness,
-			     size,
-			     widget->allocation.height - widget->style->ythickness * 2);
-	      return;
+	      gtk_progress_bar_paint_continuous (pbar, amount, orientation);
+
+	      if (GTK_PROGRESS (pbar)->show_text)
+		gtk_progress_bar_paint_text (pbar, amount, orientation);
 	    }
 	  else
-	    {
-	      size = MAX (2, widget->allocation.height / pbar->activity_blocks);
-	      
-	      gtk_paint_box (widget->style,
-			     progress->offscreen_pixmap,
-			     GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-			     NULL, widget, "bar",
-			     widget->style->xthickness,
-			     pbar->activity_pos,
-			     widget->allocation.width - widget->style->xthickness * 2,
-			     size);
-	      return;
-	    }
+	    gtk_progress_bar_paint_discrete (pbar, orientation);
 	}
-      
-      amount = percentage * space;
-      
-      if (amount > 0)
-	{
-	  switch (pbar->orientation)
-	    {
-	      
-	    case GTK_PROGRESS_LEFT_TO_RIGHT:
-	      
-	      if (pbar->bar_style == GTK_PROGRESS_CONTINUOUS)
-		{
-		  gtk_paint_box (widget->style,
-				 progress->offscreen_pixmap,
-				 GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-				 NULL, widget, "bar",
-				 widget->style->xthickness,
-				 widget->style->ythickness,
-				 amount,
-				 widget->allocation.height - widget->style->ythickness * 2);
-		}
-	      else
-		{
-		  x = widget->style->xthickness;
-		  
-		  for (i = 0; i <= pbar->in_block; i++)
-		    {
-		      block_delta = (((i + 1) * space) / pbar->blocks)
-			- ((i * space) / pbar->blocks);
-		      
-		      gtk_paint_box (widget->style,
-				     progress->offscreen_pixmap,
-				     GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-				     NULL, widget, "bar",
-				     x,
-				     widget->style->ythickness,
-				     block_delta,
-				     widget->allocation.height - widget->style->ythickness * 2);
-		      
-		      x +=  block_delta;
-		    }
-		}
-	      break;
-	      
-	    case GTK_PROGRESS_RIGHT_TO_LEFT:
-	      
-	      if (pbar->bar_style == GTK_PROGRESS_CONTINUOUS)
-		{
-		  gtk_paint_box (widget->style,
-				 progress->offscreen_pixmap,
-				 GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-				 NULL, widget, "bar",
-				 widget->allocation.width - 
-				 widget->style->xthickness - amount,
-				 widget->style->ythickness,
-				 amount,
-				 widget->allocation.height -
-				 widget->style->ythickness * 2);
-		}
-	      else
-		{
-		  x = widget->allocation.width - 
-		    widget->style->xthickness;
-		  
-		  for (i = 0; i <= pbar->in_block; i++)
-		    {
-		      block_delta = (((i + 1) * space) / pbar->blocks) -
-			((i * space) / pbar->blocks);
-		      
-		      x -=  block_delta;
-		      
-		      gtk_paint_box (widget->style,
-				     progress->offscreen_pixmap,
-				     GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-				     NULL, widget, "bar",
-				     x,
-				     widget->style->ythickness,
-				     block_delta,
-				     widget->allocation.height -
-				     widget->style->ythickness * 2);
-		    }
-		}
-	      break;
+    }
+}
 
-	    case GTK_PROGRESS_BOTTOM_TO_TOP:
+static void
+gtk_progress_bar_set_bar_style_internal (GtkProgressBar     *pbar,
+					 GtkProgressBarStyle bar_style)
+{
+  g_return_if_fail (GTK_IS_PROGRESS_BAR (pbar));
 
-	      if (pbar->bar_style == GTK_PROGRESS_CONTINUOUS)
-		{
-		  gtk_paint_box (widget->style,
-				 progress->offscreen_pixmap,
-				 GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-				 NULL, widget, "bar",
-				 widget->style->xthickness,
-				 widget->allocation.height - 
-				 widget->style->ythickness - amount,
-				 widget->allocation.width -
-				 widget->style->xthickness * 2,
-				 amount);
-		}
-	      else
-		{
-		  y = widget->allocation.height - 
-		    widget->style->ythickness;
-		  
-		  for (i = 0; i <= pbar->in_block; i++)
-		    {
-		      block_delta = (((i + 1) * space) / pbar->blocks) -
-			((i * space) / pbar->blocks);
-		      
-		      y -= block_delta;
-		      
-		      gtk_paint_box (widget->style,
-				     progress->offscreen_pixmap,
-				     GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-				     NULL, widget, "bar",
-				     widget->style->xthickness,
-				     y,
-				     widget->allocation.width - 
-				     widget->style->xthickness * 2,
-				     block_delta);
-		    }
-		}
-	      break;
-	      
-	    case GTK_PROGRESS_TOP_TO_BOTTOM:
-	      
-	      if (pbar->bar_style == GTK_PROGRESS_CONTINUOUS)
-		{
-		  gtk_paint_box (widget->style,
-				 progress->offscreen_pixmap,
-				 GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-				 NULL, widget, "bar",
-				 widget->style->xthickness,
-				 widget->style->ythickness,
-				 widget->allocation.width -
-				 widget->style->xthickness * 2,
-				 amount);
-		}
-	      else
-		{
-		  y = widget->style->ythickness;
-		  
-		  for (i = 0; i <= pbar->in_block; i++)
-		    {
-		      
-		      block_delta = (((i + 1) * space) / pbar->blocks)
-			- ((i * space) / pbar->blocks);
-		      
-		      gtk_paint_box (widget->style,
-				     progress->offscreen_pixmap,
-				     GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-				     NULL, widget, "bar",
-				     widget->style->xthickness,
-				     y,
-				     widget->allocation.width -
-				     widget->style->xthickness * 2,
-				     block_delta);
-		      
-		      y += block_delta;
-		    }
-		}
-	      break;
-	      
-	    default:
-	      break;
-	    }
-	}
-      
-      if (progress->show_text && pbar->bar_style != GTK_PROGRESS_DISCRETE)
-	{
-	  gint x;
-	  gint y;
-	  gchar *buf;
-	  GdkRectangle rect;
-	  PangoLayout *layout;
-	  PangoRectangle logical_rect;
+  if (pbar->bar_style != bar_style)
+    {
+      pbar->bar_style = bar_style;
 
-	  buf = gtk_progress_get_current_text (progress);
+      if (GTK_WIDGET_DRAWABLE (GTK_WIDGET (pbar)))
+	gtk_widget_queue_resize (GTK_WIDGET (pbar));
 
-	  layout = gtk_widget_create_pango_layout (widget, buf);
-	  pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
-	  
-	  x = widget->style->xthickness + 1 + 
-	    (widget->allocation.width - 2 * widget->style->xthickness -
-	     3 - logical_rect.width)
-	    * progress->x_align; 
+      g_object_notify (G_OBJECT (pbar), "bar_style");
+    }
+}
 
-	  y = widget->style->ythickness + 1 +
-	    (widget->allocation.height - 2 * widget->style->ythickness -
-	     3 - logical_rect.height)
-	    * progress->y_align;
+static void
+gtk_progress_bar_set_discrete_blocks_internal (GtkProgressBar *pbar,
+					       guint           blocks)
+{
+  g_return_if_fail (GTK_IS_PROGRESS_BAR (pbar));
+  g_return_if_fail (blocks > 1);
 
-	  rect.x = widget->style->xthickness + 1;
-	  rect.y = widget->style->ythickness + 1;
-	  rect.width = widget->allocation.width -
-	    2 * widget->style->xthickness - 3;
-	  rect.height = widget->allocation.height -
-	    2 * widget->style->ythickness - 3;
-      
-          gtk_paint_layout (widget->style,
-                            progress->offscreen_pixmap,
-                            GTK_WIDGET_STATE (widget),
-			    FALSE,
-                            &rect,
-                            widget,
-                            "progressbar",
-                            x, y,
-                            layout);
+  if (pbar->blocks != blocks)
+    {
+      pbar->blocks = blocks;
 
-          g_object_unref (G_OBJECT (layout));
-	  g_free (buf);
- 	}
+      if (GTK_WIDGET_DRAWABLE (GTK_WIDGET (pbar)))
+	gtk_widget_queue_resize (GTK_WIDGET (pbar));
+
+      g_object_notify (G_OBJECT (pbar), "discrete_blocks");
+    }
+}
+
+static void
+gtk_progress_bar_set_activity_step_internal (GtkProgressBar *pbar,
+					     guint           step)
+{
+  g_return_if_fail (GTK_IS_PROGRESS_BAR (pbar));
+
+  if (pbar->activity_step != step)
+    {
+      pbar->activity_step = step;
+      g_object_notify (G_OBJECT (pbar), "activity_step");
+    }
+}
+
+static void
+gtk_progress_bar_set_activity_blocks_internal (GtkProgressBar *pbar,
+					       guint           blocks)
+{
+  g_return_if_fail (GTK_IS_PROGRESS_BAR (pbar));
+  g_return_if_fail (blocks > 1);
+
+  if (pbar->activity_blocks != blocks)
+    {
+      pbar->activity_blocks = blocks;
+      g_object_notify (G_OBJECT (pbar), "activity_blocks");
     }
 }
 
@@ -913,8 +999,11 @@ gtk_progress_bar_set_text (GtkProgressBar *pbar,
   gtk_progress_set_show_text (GTK_PROGRESS (pbar), text && *text);
   gtk_progress_set_format_string (GTK_PROGRESS (pbar), text);
   
-  /* We don't support formats in this interface */
-  GTK_PROGRESS (pbar)->use_text_format = FALSE;
+  /* We don't support formats in this interface, but turn
+   * them back on for NULL, which should put us back to
+   * the initial state.
+   */
+  GTK_PROGRESS (pbar)->use_text_format = (text == NULL);
   
   g_object_notify (G_OBJECT (pbar), "text");
 }
@@ -1053,15 +1142,7 @@ gtk_progress_bar_set_bar_style (GtkProgressBar     *pbar,
 {
   g_return_if_fail (GTK_IS_PROGRESS_BAR (pbar));
 
-  if (pbar->bar_style != bar_style)
-    {
-      pbar->bar_style = bar_style;
-
-      if (GTK_WIDGET_DRAWABLE (GTK_WIDGET (pbar)))
-	gtk_widget_queue_resize (GTK_WIDGET (pbar));
-
-      g_object_notify (G_OBJECT (pbar), "bar_style");
-    }
+  gtk_progress_bar_set_bar_style_internal (pbar, bar_style);
 }
 
 void
@@ -1071,15 +1152,7 @@ gtk_progress_bar_set_discrete_blocks (GtkProgressBar *pbar,
   g_return_if_fail (GTK_IS_PROGRESS_BAR (pbar));
   g_return_if_fail (blocks > 1);
 
-  if (pbar->blocks != blocks)
-    {
-      pbar->blocks = blocks;
-
-      if (GTK_WIDGET_DRAWABLE (GTK_WIDGET (pbar)))
-	gtk_widget_queue_resize (GTK_WIDGET (pbar));
-
-      g_object_notify (G_OBJECT (pbar), "discrete_blocks");
-    }
+  gtk_progress_bar_set_discrete_blocks_internal (pbar, blocks);
 }
 
 void
@@ -1088,11 +1161,7 @@ gtk_progress_bar_set_activity_step (GtkProgressBar *pbar,
 {
   g_return_if_fail (GTK_IS_PROGRESS_BAR (pbar));
 
-  if (pbar->activity_step != step)
-    {
-      pbar->activity_step = step;
-      g_object_notify (G_OBJECT (pbar), "activity_step");
-    }
+  gtk_progress_bar_set_activity_step_internal (pbar, step);
 }
 
 void
@@ -1102,9 +1171,5 @@ gtk_progress_bar_set_activity_blocks (GtkProgressBar *pbar,
   g_return_if_fail (GTK_IS_PROGRESS_BAR (pbar));
   g_return_if_fail (blocks > 1);
 
-  if (pbar->activity_blocks != blocks)
-    {
-      pbar->activity_blocks = blocks;
-      g_object_notify (G_OBJECT (pbar), "activity_blocks");
-    }
+  gtk_progress_bar_set_activity_blocks_internal (pbar, blocks);
 }
