@@ -43,6 +43,13 @@ typedef enum
   GTK_TREE_VIEW_IN_COLUMN_DRAG = 1 << 7
 } GtkTreeViewFlags;
 
+typedef enum
+{
+  GTK_TREE_SELECT_MODE_TOGGLE = 1 << 0,
+  GTK_TREE_SELECT_MODE_EXTEND = 1 << 1
+}
+GtkTreeSelectMode;
+
 enum
 {
   DRAG_COLUMN_WINDOW_STATE_UNSET = 0,
@@ -184,6 +191,9 @@ struct _GtkTreeViewPrivate
   /* hint to display rows in alternating colors */
   guint has_rules : 1;
   guint mark_rows_col_dirty : 1;
+
+  guint ctrl_pressed : 1;
+  guint shift_pressed : 1;
   
   /* interactive search */
   guint enable_search : 1;
@@ -195,6 +205,21 @@ struct _GtkTreeViewPrivate
   GtkDestroyNotify search_destroy;
 };
 
+/* cool ABI compat hack */
+#define GTK_CELL_RENDERER_INFO_KEY "gtk-cell-renderer-info"
+
+typedef struct _GtkCellRendererInfo GtkCellRendererInfo;
+struct _GtkCellRendererInfo
+{
+  GdkColor cell_background;
+
+  /* text renderer */
+  gulong focus_out_id;
+
+  /* toggle renderer */
+  guint inconsistent :1;
+};
+
 #ifdef __GNUC__
 
 #define TREE_VIEW_INTERNAL_ASSERT(expr, ret)     G_STMT_START{          \
@@ -272,7 +297,7 @@ void         _gtk_tree_selection_internal_select_node (GtkTreeSelection  *select
 						       GtkRBNode         *node,
 						       GtkRBTree         *tree,
 						       GtkTreePath       *path,
-						       GdkModifierType    state,
+                                                       GtkTreeSelectMode  mode,
 						       gboolean           override_browse_mode);
 gboolean     _gtk_tree_view_find_node                 (GtkTreeView       *tree_view,
 						       GtkTreePath       *path,
@@ -318,8 +343,10 @@ void             _gtk_tree_view_column_autosize          (GtkTreeView       *tre
 							  GtkTreeViewColumn *column);
 
 gboolean         _gtk_tree_view_column_has_editable_cell (GtkTreeViewColumn *column);
-GtkCellRenderer *_gtk_tree_view_column_get_editable_cell (GtkTreeViewColumn *column);
+GtkCellRenderer *_gtk_tree_view_column_get_edited_cell   (GtkTreeViewColumn *column);
 gint             _gtk_tree_view_column_count_special_cells (GtkTreeViewColumn *column);
+GtkCellRenderer *_gtk_tree_view_column_get_cell_at_pos   (GtkTreeViewColumn *column,
+							  gint               x);
 
 GtkTreeSelection* _gtk_tree_selection_new                (void);
 GtkTreeSelection* _gtk_tree_selection_new_with_tree_view (GtkTreeView      *tree_view);
@@ -333,7 +360,9 @@ void		  _gtk_tree_view_column_cell_render      (GtkTreeViewColumn *tree_column,
 							  GdkRectangle      *expose_area,
 							  guint              flags);
 gboolean	  _gtk_tree_view_column_cell_focus       (GtkTreeViewColumn *tree_column,
-							  gint               direction);
+							  gint               direction,
+							  gboolean           left,
+							  gboolean           right);
 void		  _gtk_tree_view_column_cell_draw_focus  (GtkTreeViewColumn *tree_column,
 							  GdkWindow         *window,
 							  GdkRectangle      *background_area,
