@@ -90,11 +90,11 @@ typedef enum
  * Macros for testing whether `widget' or `klass' are of type GTK_TYPE_WIDGET.
  */
 #define GTK_TYPE_WIDGET			  (gtk_widget_get_type ())
-#define GTK_WIDGET(widget)		  (GTK_CHECK_CAST ((widget), GTK_TYPE_WIDGET, GtkWidget))
-#define GTK_WIDGET_CLASS(klass)		  (GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_WIDGET, GtkWidgetClass))
-#define GTK_IS_WIDGET(widget)		  (GTK_CHECK_TYPE ((widget), GTK_TYPE_WIDGET))
-#define GTK_IS_WIDGET_CLASS(klass)	  (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_WIDGET))
-#define GTK_WIDGET_GET_CLASS(obj)         (GTK_CHECK_GET_CLASS ((obj), GTK_TYPE_WIDGET, GtkWidgetClass))
+#define GTK_WIDGET(widget)		  (G_TYPE_CHECK_INSTANCE_CAST ((widget), GTK_TYPE_WIDGET, GtkWidget))
+#define GTK_WIDGET_CLASS(klass)		  (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_WIDGET, GtkWidgetClass))
+#define GTK_IS_WIDGET(widget)		  (G_TYPE_CHECK_INSTANCE_TYPE ((widget), GTK_TYPE_WIDGET))
+#define GTK_IS_WIDGET_CLASS(klass)	  (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_WIDGET))
+#define GTK_WIDGET_GET_CLASS(obj)         (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_WIDGET, GtkWidgetClass))
 
 /* Macros for extracting various fields from GtkWidget and GtkWidgetClass.
  */
@@ -131,6 +131,7 @@ typedef enum
 #define GTK_WIDGET_SET_FLAGS(wid,flag)	  G_STMT_START{ (GTK_WIDGET_FLAGS (wid) |= (flag)); }G_STMT_END
 #define GTK_WIDGET_UNSET_FLAGS(wid,flag)  G_STMT_START{ (GTK_WIDGET_FLAGS (wid) &= ~(flag)); }G_STMT_END
 
+#define GTK_TYPE_REQUISITION              (gtk_requisition_get_type ())
 
 /* forward declaration to avoid excessive includes (and concurrent includes)
  */
@@ -140,6 +141,7 @@ typedef struct _GtkSelectionData   GtkSelectionData;
 typedef struct _GtkWidgetClass	   GtkWidgetClass;
 typedef struct _GtkWidgetAuxInfo   GtkWidgetAuxInfo;
 typedef struct _GtkWidgetShapeInfo GtkWidgetShapeInfo;
+typedef struct _GtkClipboard	   GtkClipboard;
 typedef void     (*GtkCallback)        (GtkWidget        *widget,
 					gpointer	  data);
 
@@ -195,6 +197,8 @@ struct _GtkWidget
    */
   gchar *name;
   
+  /*< public >*/
+
   /* The style for the widget. The style contains the
    *  colors the widget should be drawn in for each state
    *  along with graphics contexts used to draw with and
@@ -230,20 +234,15 @@ struct _GtkWidgetClass
    *  pointer.
    */
   GtkObjectClass parent_class;
+
+  /*< public >*/
   
-  /* The signal to emit when a widget of this class is activated,
-   * gtk_widget_activate() handles the emission.
-   * Implementation of this signal is optional.
-   */
   guint activate_signal;
 
-  /* This signal is emitted  when a widget of this class is added
-   * to a scrolling aware parent, gtk_widget_set_scroll_adjustments()
-   * handles the emission.
-   * Implementation of this signal is optional.
-   */
   guint set_scroll_adjustments_signal;
 
+  /*< private >*/
+  
   /* seldomly overidden */
   void (*dispatch_child_properties_changed) (GtkWidget   *widget,
 					     guint        n_pspecs,
@@ -346,10 +345,10 @@ struct _GtkWidgetClass
   void (* selection_get)           (GtkWidget          *widget,
 				    GtkSelectionData   *selection_data,
 				    guint               info,
-				    guint               time);
+				    guint               time_);
   void (* selection_received)      (GtkWidget          *widget,
 				    GtkSelectionData   *selection_data,
-				    guint               time);
+				    guint               time_);
 
   /* Source side drag signals */
   void (* drag_begin)	           (GtkWidget	       *widget,
@@ -360,52 +359,57 @@ struct _GtkWidgetClass
 				    GdkDragContext     *context,
 				    GtkSelectionData   *selection_data,
 				    guint               info,
-				    guint               time);
+				    guint               time_);
   void (* drag_data_delete)        (GtkWidget	       *widget,
 				    GdkDragContext     *context);
 
   /* Target side drag signals */
   void (* drag_leave)	           (GtkWidget	       *widget,
 				    GdkDragContext     *context,
-				    guint               time);
+				    guint               time_);
   gboolean (* drag_motion)         (GtkWidget	       *widget,
 				    GdkDragContext     *context,
 				    gint                x,
 				    gint                y,
-				    guint               time);
+				    guint               time_);
   gboolean (* drag_drop)           (GtkWidget	       *widget,
 				    GdkDragContext     *context,
 				    gint                x,
 				    gint                y,
-				    guint               time);
+				    guint               time_);
   void (* drag_data_received)      (GtkWidget          *widget,
 				    GdkDragContext     *context,
 				    gint                x,
 				    gint                y,
 				    GtkSelectionData   *selection_data,
 				    guint               info,
-				    guint               time);
+				    guint               time_);
 
   /* Signals used only for keybindings */
-  void (* popup_menu)              (GtkWidget          *widget);
+  gboolean (* popup_menu)          (GtkWidget          *widget);
 
   /* If a widget has multiple tooltips/whatsthis, it should show the
    * one for the current focus location, or if that doesn't make
    * sense, should cycle through them showing each tip alongside
    * whatever piece of the widget it applies to.
    */
-  void (* show_help)               (GtkWidget          *widget,
+  gboolean (* show_help)           (GtkWidget          *widget,
                                     GtkWidgetHelpType   help_type);
   
   /* accessibility support 
    */
   AtkObject*   (* get_accessible)  (GtkWidget          *widget);
 
+  void (* screen_changed) (GtkWidget *widget,
+			   GdkScreen *previous_screen);
   /* Padding for future expansion */
-  GtkFunction pad1;
-  GtkFunction pad2;
-  GtkFunction pad3;
-  GtkFunction pad4;
+  void (*_gtk_reserved1) (void);
+  void (*_gtk_reserved2) (void);
+  void (*_gtk_reserved3) (void);
+  void (*_gtk_reserved4) (void);
+  void (*_gtk_reserved5) (void);
+  void (*_gtk_reserved6) (void);
+  void (*_gtk_reserved7) (void);
 };
 
 struct _GtkWidgetAuxInfo
@@ -425,8 +429,8 @@ struct _GtkWidgetShapeInfo
   GdkBitmap *shape_mask;
 };
 
-GtkType	   gtk_widget_get_type		  (void) G_GNUC_CONST;
-GtkWidget* gtk_widget_new		  (GtkType		type,
+GType	   gtk_widget_get_type		  (void) G_GNUC_CONST;
+GtkWidget* gtk_widget_new		  (GType		type,
 					   const gchar	       *first_property_name,
 					   ...);
 GtkWidget* gtk_widget_ref		  (GtkWidget	       *widget);
@@ -491,7 +495,8 @@ gboolean   gtk_widget_remove_accelerator  (GtkWidget           *widget,
 void       gtk_widget_set_accel_path      (GtkWidget           *widget,
 					   const gchar         *accel_path,
 					   GtkAccelGroup       *accel_group);
-const gchar* _gtk_widget_get_accel_path   (GtkWidget           *widget);
+const gchar* _gtk_widget_get_accel_path   (GtkWidget           *widget,
+					   gboolean	       *locked);
 GList*     gtk_widget_list_accel_closures (GtkWidget	       *widget);
 gboolean   gtk_widget_mnemonic_activate   (GtkWidget           *widget,
 					   gboolean             group_cycling);
@@ -545,6 +550,7 @@ gboolean              gtk_widget_get_child_visible      (GtkWidget    *widget);
 
 GtkWidget *gtk_widget_get_parent          (GtkWidget           *widget);
 GdkWindow *gtk_widget_get_parent_window	  (GtkWidget	       *widget);
+
 gboolean   gtk_widget_child_focus         (GtkWidget           *widget,
                                            GtkDirectionType     direction);
 
@@ -573,11 +579,17 @@ void	   gtk_widget_set_extension_events (GtkWidget		*widget,
 GdkExtensionMode gtk_widget_get_extension_events (GtkWidget	*widget);
 GtkWidget*   gtk_widget_get_toplevel	(GtkWidget	*widget);
 GtkWidget*   gtk_widget_get_ancestor	(GtkWidget	*widget,
-					 GtkType	widget_type);
+					 GType		 widget_type);
 GdkColormap* gtk_widget_get_colormap	(GtkWidget	*widget);
 GdkVisual*   gtk_widget_get_visual	(GtkWidget	*widget);
 
-GtkSettings* gtk_widget_get_settings    (GtkWidget      *widget);
+GdkScreen *   gtk_widget_get_screen      (GtkWidget *widget);
+gboolean      gtk_widget_has_screen      (GtkWidget *widget);
+GdkDisplay *  gtk_widget_get_display     (GtkWidget *widget);
+GdkWindow *   gtk_widget_get_root_window (GtkWidget *widget);
+GtkSettings*  gtk_widget_get_settings    (GtkWidget *widget);
+GtkClipboard *gtk_widget_get_clipboard   (GtkWidget *widget,
+					  GdkAtom    selection);
 
 #ifndef GTK_DISABLE_DEPRECATED
 #define gtk_widget_set_visual(widget,visual)  ((void) 0)
@@ -683,6 +695,10 @@ void gtk_widget_class_install_style_property        (GtkWidgetClass     *klass,
 void gtk_widget_class_install_style_property_parser (GtkWidgetClass     *klass,
 						     GParamSpec         *pspec,
 						     GtkRcPropertyParser parser);
+GParamSpec*  gtk_widget_class_find_style_property   (GtkWidgetClass     *klass,
+						     const gchar        *property_name);
+GParamSpec** gtk_widget_class_list_style_properties (GtkWidgetClass     *klass,
+						     guint              *n_properties);
 void gtk_widget_style_get_property (GtkWidget	     *widget,
 				    const gchar    *property_name,
 				    GValue	     *value);
@@ -698,8 +714,10 @@ void gtk_widget_style_get          (GtkWidget	     *widget,
  */
 void	     gtk_widget_set_default_colormap (GdkColormap *colormap);
 GtkStyle*    gtk_widget_get_default_style    (void);
+#ifndef GDK_MULTIHEAD_SAFE
 GdkColormap* gtk_widget_get_default_colormap (void);
 GdkVisual*   gtk_widget_get_default_visual   (void);
+#endif
 
 /* Functions for setting directionality for widgets
  */
@@ -733,8 +751,9 @@ void	     gtk_widget_class_path	   (GtkWidget *widget,
 					    gchar    **path,
 					    gchar    **path_reversed);
 
-GtkRequisition *gtk_requisition_copy (const GtkRequisition *requisition);
-void            gtk_requisition_free (GtkRequisition       *requisition);
+GType           gtk_requisition_get_type (void);
+GtkRequisition *gtk_requisition_copy     (const GtkRequisition *requisition);
+void            gtk_requisition_free     (GtkRequisition       *requisition);
 
 #if	defined (GTK_TRACE_OBJECTS) && defined (__GNUC__)
 #  define gtk_widget_ref gtk_object_ref
@@ -745,9 +764,12 @@ GtkWidgetAuxInfo *_gtk_widget_get_aux_info                (GtkWidget    *widget,
 							   gboolean      create);
 void              _gtk_widget_propagate_hierarchy_changed (GtkWidget    *widget,
 							   GtkWidget    *previous_toplevel);
+void              _gtk_widget_propagate_screen_changed    (GtkWidget    *widget,
+							   GdkScreen    *previous_screen);
 
 GdkColormap* _gtk_widget_peek_colormap (void);
 
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
